@@ -9,7 +9,7 @@
 # You should have received a copy of the GNU General Public License along with seacarb; if not, write to the Free Software Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 #
 "K1" <-
-function(S=35,T=25,P=0,k1k2='x',pHscale="T",kSWS2scale=0,ktotal2SWS_P0=0,warn="y")
+function(S=35,T=25,P=0,k1k2='x',pHscale="T",kSWS2scale="x",ktotal2SWS_P0="x",warn="y")
 {
 
     nK <- max(length(S), length(T), length(P), length(k1k2), length(pHscale), length(kSWS2scale) ,length(ktotal2SWS_P0))
@@ -56,6 +56,35 @@ function(S=35,T=25,P=0,k1k2='x',pHscale="T",kSWS2scale=0,ktotal2SWS_P0=0,warn="y
     pHsc[is_l] <- "T"
 
 
+    # --------------------- K1 ---------------------------------------
+    #   first acidity constant:
+    #   [H^+] [HCO_3^-] / [CO2] = K_1
+    #
+    #     Papadimitriou et al (2018)  in Geochimica et Cosmochimica Acta 220 (2018) 55–70
+    #
+    #   pH-scale: 'total'. mol/kg-soln
+    is_p18 <- k1k2 == "p18"
+    Sp18 <- S[is_p18]
+    pK1 <- -176.48 + 6.14528*sqrt(Sp18) - 0.127714*Sp18 + 7.396e-5*Sp18*Sp18 +
+            (9914.37 - 622.886*sqrt(Sp18) + 29.714*Sp18)/TK[is_p18] +
+            (26.05129 - 0.666812*sqrt(Sp18)) * log(TK[is_p18])
+    K1[is_p18]<- 10^(-pK1)
+    pHsc[is_p18] <- "T"
+
+
+    # --------------------- K1 ---------------------------------------
+    #   first acidity constant:
+    #   [H^+] [HCO_3^-] / [CO2] = K_1
+    #
+    #     Sulpis et al (2020)  in Ocean Science, 16, 847–862, 2020
+    #
+    #   pH-scale: 'total'. mol/kg-soln
+    is_s20 <- k1k2 == "s20"
+    pK1 <- 8510.63/TK[is_s20] - 172.4493 + 26.32996*log(TK[is_s20]) - 0.011555*S[is_s20] + 0.0001152*S[is_s20]*S[is_s20]
+    K1[is_s20]<- 10^(-pK1)
+    pHsc[is_s20] <- "T"
+
+
     # --------------------- K1 ---------------------------------------
     #   first acidity constant:
     #   [H^+] [HCO_3^-] / [CO2] = K_1
@@ -71,6 +100,34 @@ function(S=35,T=25,P=0,k1k2='x',pHscale="T",kSWS2scale=0,ktotal2SWS_P0=0,warn="y
     K1[is_r] <- exp(lnK1roy)
     pHsc[is_r] <- "T"
 
+
+    # --------------------- K1 ---------------------------------------
+    #   first acidity constant:
+    #   [H^+] [HCO_3^-] / [CO2] = K_1
+    #
+    #   Mojica Prieto and Millero (2002)
+    #
+    #   pH-scale: 'seawater'. mol/kg-soln
+    is_mp2 <- k1k2 == "mp2"
+    pK1 <- -43.6977 - 0.0129037 * S[is_mp2] + 1.364e-4 * S[is_mp2]*S[is_mp2] + 2885.378/TK[is_mp2] + 7.045159 * log(TK[is_mp2])
+    K1[is_mp2]<- 10^(-pK1)
+    pHsc[is_mp2] <- "SWS"
+
+    
+    # --------------------- K1 ---------------------------------------
+    #   first acidity constant:
+    #   [H^+] [HCO_3^-] / [CO2] = K_1
+    #
+    # Millero et al., 2002. Deep-Sea Res. I (49) 1705-1723.
+    # Calculated from overdetermined WOCE-era field measurements
+    # This is from page 1715
+    #
+    #   pH-scale: 'seawater'. mol/kg-soln
+    is_m02 <- k1k2 == "m02"
+    pK1 =  6.359 - 0.00664 * S[is_m02] - 0.01322 * T[is_m02] + 4.989e-5 * T[is_m02]*T[is_m02]
+    K1[is_m02]<- 10^(-pK1)
+    pHsc[is_m02] <- "SWS"
+    
         
     # --------------------- K1 ---------------------------------------
     #   first acidity constant:
@@ -173,7 +230,7 @@ function(S=35,T=25,P=0,k1k2='x',pHscale="T",kSWS2scale=0,ktotal2SWS_P0=0,warn="y
     {
         ##------------- Convert from total to SWS scale
         # if correction factor (from Total scale to seawater at P=0) not given
-        if (missing(ktotal2SWS_P0))
+        if (missing(ktotal2SWS_P0) || ktotal2SWS_P0[1] == "x")
         {
             # Compute it
             ktotal2SWS_P0  <- rep(1.0,nK)
@@ -200,11 +257,9 @@ function(S=35,T=25,P=0,k1k2='x',pHscale="T",kSWS2scale=0,ktotal2SWS_P0=0,warn="y
     if (length(i_press) > 0)
     {
         # Call Pcorrect() on SWS scale
-	# issue (Orr): Why are the last 2 argupments set to one here?
+	# last 2 argupments are conversion factors: they are set to one since no PH scale conversion needed
         K1[i_press] <- Pcorrect(Kvalue=K1[i_press], Ktype="K1", T=T[i_press], 
              S=S[i_press], P=P[i_press], pHscale=pHsc[i_press], 1., 1., warn=warn)
-        #K1[i_press] <- Pcorrect(Kvalue=K1[i_press], Ktype="K1", T=T[i_press], 
-        #    S=S[i_press], P=P[i_press], pHscale=pHsc[i_press], warn=warn)
     }
 
 
@@ -220,7 +275,7 @@ function(S=35,T=25,P=0,k1k2='x',pHscale="T",kSWS2scale=0,ktotal2SWS_P0=0,warn="y
     if (any(convert))
     {
         # if pH scale correction factor not given
-        if (missing(kSWS2scale))
+        if (missing(kSWS2scale) || kSWS2scale[1] == "x")
         {
             # Compute it
             kSWS2scale <- rep(1.0,nK)
@@ -248,7 +303,11 @@ function(S=35,T=25,P=0,k1k2='x',pHscale="T",kSWS2scale=0,ktotal2SWS_P0=0,warn="y
 
     is_w <- warn == "y"
 
-    if (any(is_w & is_l & (T>35 | T<2 | S<19 | S>43))  || any(is_w & is_r & (T>45 | S<5 | S>45)) )
+    if (   any (is_w & is_l & (T>35 | T<2 | S<19 | S>43))  || any (is_w & is_r & (T<0 | T>45 | S<5 | S>45)) 
+        || any (is_w & is_mp2 & (T<0 | T>45 | S<5 | S>42)) || any (is_w & is_m02 & (T<(-1.6) | T>35 | S<34 | S>37))
+        || any (is_w & is_m06 & (T<1 | T>50 | S<0.1 | S>50)) 
+        || any (is_w & (is_m10 | is_w14) & (T<0 | T>50 | S<1 | S>50))
+        || any (is_w & is_p18 & (T<(-6) | T>25 | S<33 | S>100)) || any (is_w & is_s20 & (T<(-1.7) | T>31.8 | S<30.7 | S>37.6)) )
         warning("S and/or T is outside the range of validity of the formulation chosen for K1.")
 
     if (any(is_w & (T>50 | S>50))) 
@@ -256,11 +315,15 @@ function(S=35,T=25,P=0,k1k2='x',pHscale="T",kSWS2scale=0,ktotal2SWS_P0=0,warn="y
 
     ##---------------Attributes
     method <- rep(NA, nK)
+    method[is_mp2] <- "Mojica Prieto et al. (2002)"
+    method[is_m02] <- "Millero et al. (2002)"
     method[is_m06] <- "Millero et al. (2006)"
     method[is_m10] <- "Millero (2010)"
     method[is_w14] <- "Waters et al. (2014)"
     method[is_r]   <- "Roy et al. (1993)"
-    method[! (is_m06 | is_m10 | is_w14 | is_r) ] <- "Luecker et al. (2000)"
+    method[is_p18] <- "Papadimitriou et al. (2018)"
+    method[is_s20] <- "Sulpis et al. (2020)"
+    method[! (is_mp2 | is_m02 | is_m06 | is_m10 | is_w14 | is_r | is_p18 | is_s20) ] <- "Luecker et al. (2000)"
 
     attr(K1,"unit") <- "mol/kg-soln"
     attr(K1,"pH scale") <- pHlabel
